//------------------------------------------------------------------------------------------------

//
// File: polygon.h
// Author: Roman Lysecky
// Date Created: 28 November 2011
// 
// Description: Declaration of singly-linked list of LineSegment's representing a polygon.
// 
// Copyright: Roman Lysecky, Electrical and Computer Engineering, University of Arizona, 2011.
// 
// 

//------------------------------------------------------------------------------------------------

#ifndef POLYGON_H
#define POLYGON_H

//------------------------------------------------------------------------------------------------

#include <iostream>
#include "linesegment.h"

//------------------------------------------------------------------------------------------------

using namespace std;

//------------------------------------------------------------------------------------------------

class Polygon {
public:
    // Polygon constructor: initialize polygon to empty list 
    Polygon();
    
    // 
    // Polygon destructor: destroys the list by freeing all memory allocated for each line 
    // segment returning the polygon to an empty list
    //
    ~Polygon();
    
public:
    // 
    // inserts new line segment specified by (x1,y1), (x2, y2) into list after the given list 
    // element location. If the list element location is NULL, inserts line segment at head of list 
    //
    void insert_segment_next(float x1, float y1, float x2, float y2, LineSegment *location=0);
    
    // 
    // removes the line segment after the the given list element location. 
    // If the list element location is NULL, removes the head of the list
    //
    void remove_segment_next(LineSegment *location=0);
    
    // returns the number of line segments within the polygon */
    int num_segments();
    
    // returns pointer to next line segment within the list */
    LineSegment *next_segment(LineSegment *segment);
    
    // 
    // checks to determine if the specified polygon is a valid closed form polygon in 
    // which all segments are connected to another line segment
    //
    bool is_valid();
    
    // prints list of line segments within polygon 
    void display();

    // Overloading the output stream operator so we can display the line segment
    // using C++ stream such as cout
    friend ostream& operator<< (ostream &os, Polygon &polygon);
    
private:
    LineSegment* find_next_connected_segment(LineSegment* fromSegment);

public:
    LineSegment *head;
    int size;
};

//------------------------------------------------------------------------------------------------

ostream& operator<< (ostream &os, Polygon &polygon);

//------------------------------------------------------------------------------------------------

#endif

//------------------------------------------------------------------------------------------------

